<?php
/**
 * Onboarding flow for themes.
 *
 * @package WPZOOM
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * WPZOOM_Onboarding Class
 *
 * @package WPZOOM
 * @subpackage Onboarding
 */
class WPZOOM_Onboarding {
	/**
	 * WPZOOM TGMPA Assistance instance.
	 *
	 * @var object
	 */
	protected $tgmpa_assistance;

	/**
	 * WPZOOM_Onboarding class instance.
	 *
	 * @access private
	 * @var object
	 */
	private static $instance;

	/**
	 * Holds configurable array of strings.
	 *
	 * Default values are added in the constructor.
	 *
	 * @since 2.0.0
	 *
	 * @var array
	 */
	public $strings = array();

	/**
	 * WP_Theme object.
	 *
	 * @var object
	 */
	public $theme;

	/**
	 * Slug name of the current theme.
	 *
	 * @var string
	 */
	public $theme_slug;

	/**
	 * HTML classname prefix.
	 *
	 * @var string
	 */
	protected $classname_prefix = 'wpz-onboard_';

	/**
	 * Initiator.
	 */
	public static function get_instance() {
		if ( ! isset( self::$instance ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Constructor.
	 */
	public function __construct() {
		// Include class utils.
		require_once WPZOOM_INC . '/components/onboarding/class-wpzoom-onboarding-utils.php';

		// Onboarding action init.
		add_action( 'wpzoom_onboarding_init', array( $this, 'set_selected_design' ) );

		$this->init();
		$this->setup_demo_import();

		// References TGMPA Assistance constructor.
		$this->init_tgmpa_instance();

		// Add extra submenu options to the admin panel’s.
		add_action( 'admin_menu', array( $this, 'admin_page' ) );

		// Include onboarding scripts/styles.
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

		// Sort plugins filter.
		add_filter( 'wpz_onboarding_sort_required_plugins', array( $this, 'sort_plugins' ) );

		// AJAX actions.
		add_action( 'wp_ajax_wpzoom_get_all_plugins', array( $this, 'get_installed_plugins_ajax' ) );
		add_action( 'wp_ajax_wpzoom_required_plugins', array( $this, 'plugin_install_ajax' ) );
		add_action( 'wp_ajax_wpzoom_get_theme_design', array( $this, 'get_theme_design_ajax' ) );

		// Create option with some value, disable the wpform redirect activation.
		update_option( 'wpforms_activation_redirect', true );

		// Onboarding Tour.
		if ( zoom_current_theme_supports( 'wpz-onboarding', 'onboarding-tour' ) ) {
			require_once WPZOOM_INC . '/components/onboarding/class-wpzoom-onboarding-tour.php';
		}
	}

	/**
	 * Adds a menu item for the theme license under the appearance menu.
	 *
	 * @since 2.0.0
	 */
	public function admin_page() {
		add_submenu_page(
			'wpzoom_options',
			__( 'Demo Content & License Key', 'wpzoom' ),
			__( 'Demo Content & License Key', 'wpzoom' ),
			'manage_options',
			'wpzoom_license',
			array( $this, 'display_admin_page' )
		);
	}

	/**
	 * Initialise the class configurations.
	 *
	 * @return void
	 */
	protected function init() {
		// Template name.
		$this->theme_slug = get_template();

		// Theme object.
		$this->theme = wp_get_theme( $this->theme_slug );

		// Load class strings.
		$this->strings = array(
			'label_quickstart'                => __( 'Quick Start', 'wpzoom' ),
			'label_license'                   => __( 'License Key', 'wpzoom' ),
			'label_wpzoom'                    => __( 'WPZOOM', 'wpzoom' ),
			'label_themes'                    => __( 'Themes', 'wpzoom' ),
			'label_plugins'                   => __( 'Plugins', 'wpzoom' ),
			'label_blog'                      => __( 'Blog', 'wpzoom' ),
			'label_support'                   => __( 'Support', 'wpzoom' ),
			/* translators: %s theme name. */
			'label_content_title'             => sprintf( __( 'Welcome to %s! &#128075;', 'wpzoom' ), $this->theme->get( 'Name' ) ),
			'label_content_intro'             => __( 'You just installed a new theme and don&rsquo;t know where to start? We&rsquo;re here to help you!', 'wpzoom' ),
			'label_license_title'             => __( 'Activate Your License Key!', 'wpzoom' ),
			/* translators: %s license account url. */
			'label_license_intro'             => sprintf( __( 'Enter your license key to enable 1-click theme updates. You can find your license in <a href="%s" target="_blank">WPZOOM Members Area → Licenses</a>.', 'wpzoom' ), esc_url( 'https://www.wpzoom.com/account/licenses/' ) ),
			'label_license_field_name'        => __( 'License Key', 'wpzoom' ),
			'label_license_note'              => __( 'Enter your license key for this theme to enable theme updates', 'wpzoom' ),
			'label_license_submit'            => __( 'Save &amp; Activate', 'wpzoom' ),
			'label_steps_title1'              => __( 'Install required plugins', 'wpzoom' ),
			'label_steps_intro1'              => __( 'To enable all the features from your theme, you&rsquo;ll need to install and activate the required plugins such as Elementor or WooCommerce, which are available for free.', 'wpzoom' ),
			'label_steps_title2'              => __( 'Choose a demo', 'wpzoom' ),
			'label_steps_intro2'              => __( 'You can choose a variation of the theme that suits your intended uses better.', 'wpzoom' ),
			'label_steps_title3'              => __( 'Import demo', 'wpzoom' ),
			/* translators: %s admin import page. */
			'label_steps_intro3'              => sprintf( __( 'Click on the button below to load the demo content. This is useful for seeing how the theme will look like when filled with content. If the importer doesn&rsquo;t work for you, try importing the demo content <a href="%s">manually</a>.', 'wpzoom' ), admin_url( 'import.php' ) ),
			'label_docs_title'                => __( 'Read documentation', 'wpzoom' ),
			'label_docs_content'              => __( 'Documentation is the place where you&rsquo;ll find the information needed to setup the theme quickly, and other details about theme-specific features.', 'wpzoom' ),
			'label_docs_button'               => __( 'Read documentation', 'wpzoom' ),
			'label_elementor_title'           => __( 'Powered by Elementor', 'wpzoom' ),
			'label_elementor_content'         => __( 'Elementor is the leading website builder platform for professionals on WordPress.', 'wpzoom' ),
			'label_elementor_button'          => __( 'Learn more', 'wpzoom' ),
			'label_assist_title'              => __( 'Need assistance?', 'wpzoom' ),
			'label_assist_content'            => __( 'Need help setting up your theme or have a question? Get in touch with our Support Team. We&rsquo;d love the opportunity to help you.', 'wpzoom' ),
			'label_assist_button'             => __( 'Open Support Desk', 'wpzoom' ),
			'label_newwp_title'               => __( 'New to WordPress?', 'wpzoom' ),
			'label_newwp_content'             => __( '20+ video tutorials that cover everything you need to know to get started using WordPress.', 'wpzoom' ),
			'label_newwp_button'              => __( 'Learn WordPress today', 'wpzoom' ),
			'label_follow_title'              => __( 'Follow WPZOOM', 'wpzoom' ),
			'label_follow_content'            => __( 'Follow us on social media for news and updates on all your theme needs.', 'wpzoom' ),
			'label_follow_button1'            => __( 'Twitter', 'wpzoom' ),
			'label_follow_button2'            => __( 'Facebook', 'wpzoom' ),
			'label_follow_button3'            => __( 'Instagram', 'wpzoom' ),
			'label_go_back'                   => __( 'Go back', 'wpzoom' ),
			'label_page_preview'              => __( 'Page preview', 'wpzoom' ),
			'label_live_preview'              => __( 'Live preview', 'wpzoom' ),
			'label_template'                  => __( 'Selected template:', 'wpzoom' ),
			'label_import_button'             => __( 'Import demo content', 'wpzoom' ),
			'label_view_pages'                => __( 'View pages', 'wpzoom' ),
			'label_selected'                  => __( 'Selected', 'wpzoom' ),
			/* translators: 1: plugin name(s). */
			'notice_can_install_required'     => _n_noop(
				'In order to import the demo content correctly, please install the following required plugin: %1$s',
				'In order to import the demo content correctly, please install the following required plugins: %1$s',
				'wpzoom'
			),
			/* translators: 1: plugin name(s). */
			'notice_can_install_recommended'  => _n_noop(
				'This theme recommends the following plugin: %1$s',
				'This theme recommends the following plugins: %1$s',
				'wpzoom'
			),
			/* translators: 1: plugin name(s). */
			'notice_ask_to_update'            => _n_noop(
				'The following plugin needs to be updated to its latest version to ensure maximum compatibility with this theme: %1$s',
				'The following plugins need to be updated to their latest version to ensure maximum compatibility with this theme: %1$s',
				'wpzoom'
			),
			/* translators: 1: plugin name(s). */
			'notice_ask_to_update_maybe'      => _n_noop(
				'There is an update available for: %1$s',
				'There are updates available for the following plugins: %1$s',
				'wpzoom'
			),
			/* translators: 1: plugin name(s). */
			'notice_can_activate_required'    => _n_noop(
				'The following required plugin is currently inactive: %1$s',
				'The following required plugins are currently inactive: %1$s',
				'wpzoom'
			),
			/* translators: 1: plugin name(s). */
			'notice_can_activate_recommended' => _n_noop(
				'The following recommended plugin is currently inactive: %1$s',
				'The following recommended plugins are currently inactive: %1$s',
				'wpzoom'
			),
			'notice_skip_recommended_link'    => _n_noop(
				'Skip recommended notice',
				'Skip recommended notices',
				'wpzoom'
			),
			'title_install_required'          => _n_noop(
				'Missing required plugin',
				'Missing required plugins',
				'wpzoom'
			),
			'title_activate_required'         => _n_noop(
				'Inactive required plugin',
				'Inactive required plugins',
				'wpzoom'
			),
			'title_install_recommended'       => _n_noop(
				'Install recommended plugin',
				'Install recommended plugins',
				'wpzoom'
			),
			'title_activate_recommended'      => _n_noop(
				'Activate recommended plugin',
				'Activate recommended plugins',
				'wpzoom'
			),
			'title_update_maybe'              => _n_noop(
				'Update plugin',
				'Update plugins',
				'wpzoom'
			),
			'title_update'                    => _n_noop(
				'Update required for plugin',
				'Update required for plugins',
				'wpzoom'
			),
			'localize_strings'                => array(
				'submit_label_normal'      => __( 'Install & activate', 'wpzoom' ),
				'submit_label_all'         => __( 'Install & activate all', 'wpzoom' ),
				'submit_label_selected'    => __( 'Install & activate selected', 'wpzoom' ),
				'submit_label_loading'     => __( 'Installing...', 'wpzoom' ),
				'preparing_preview'        => __( 'Preparing preview', 'wpzoom' ),
				'preparing_demo_content'   => __( 'Preparing demo content...', 'wpzoom' ),
				'load_demo_content'        => __( 'Load demo content', 'wpzoom' ),
				'import_demo_content'      => __( 'Import demo content', 'wpzoom' ),
				'processing_template_data' => __( 'Processing demo content', 'wpzoom' ),
				'selected'                 => __( 'Selected', 'wpzoom' ),
				'imported'                 => __( 'Imported', 'wpzoom' ),
				'select_template'          => __( 'Select', 'wpzoom' ),
				'check_label_all'          => __( 'Select all', 'wpzoom' ),
				'check_label_none'         => __( 'Unselect all', 'wpzoom' ),
				'already_installed'        => __( 'Installed', 'wpzoom' ),
				'install_failed'           => __( 'Plugin installation failed! The server returned: ', 'wpzoom' ),
				'active'                   => __( 'Active', 'wpzoom' ),
				'something_wrong'          => __( 'Something went wrong when trying to install & activate plugins! Please, try again later.', 'wpzoom' ),
				'saving_license'           => __( 'Saving & activating license key...', 'wpzoom' ),
				'deactivating_license'     => __( 'Deactivating license key...', 'wpzoom' ),
				'deactivate_license'       => __( 'Deactivate', 'wpzoom' ),
				'save_activate_license'    => __( 'Save & Activate', 'wpzoom' ),
				'save_activate_license'    => __( 'Save & Activate', 'wpzoom' ),
				'something_wrong_license'  => __( 'Something went wrong when trying to save & activate license key! Please, try again later.', 'wpzoom' ),
			),
		);

		// Maybe we don't need the activation redirect for Recipe Card Blocks plugin.
		$plugins = WPZOOM_TGMPA_Assistance::get_instance()->get_plugins();
		if ( ! empty( $plugins ) ) {
			foreach ( $plugins as $plugin ) {
				if ( 'recipe-card-blocks-by-wpzoom' === $plugin['slug'] || 'recipe-card-blocks-by-wpzoom-pro' === $plugin['slug'] ) {
					delete_option( 'wpzoom_rcb_do_activation_redirect' );
					break;
				}
			}
		}

		/**
		 * Register configuration strings, current theme data, class name prefix ...
		 *
		 * @since 2.0.0
		 */
		do_action( 'wpzoom_onboarding_init' );
	}

	/**
	 * Initialise the class WPZOOM_TGMPA_Assistance.
	 *
	 * @return void
	 */
	public function init_tgmpa_instance() {
		$this->tgmpa_assistance = WPZOOM_TGMPA_Assistance::get_instance()->tgmpa_instance();
	}

	/**
	 * Enqueue the admin styles/scripts
	 *
	 * @param string $hook_suffix The current admin page.
	 */
	public function enqueue_scripts( $hook_suffix ) {
		if ( 'wpzoom_page_wpzoom_license' !== $hook_suffix ) {
			return;
		}

		// phpcs:disable WordPress.NamingConventions.ValidVariableName.UsedPropertyNotSnakeCase
		wp_enqueue_style( 'wpzoom-options', WPZOOM::$assetsPath . '/options.css', array( 'thickbox' ), WPZOOM::$wpzoomVersion );

		wp_enqueue_style(
			'wpzoom-theme-admin-style',
			WPZOOM::get_root_uri() . '/components/onboarding/assets/css/admin.css',
			array( 'wpzoom-options' ),
			WPZOOM::$wpzoomVersion
		);

		wp_enqueue_script(
			'wpzoom-onboarding-admin-script',
			WPZOOM::get_root_uri() . '/components/onboarding/assets/js/admin.js',
			array( 'jquery' ),
			WPZOOM::$wpzoomVersion,
			true
		);
		// phpcs:enable WordPress.NamingConventions.ValidVariableName.UsedPropertyNotSnakeCase

		wp_localize_script(
			'wpzoom-onboarding-admin-script',
			'wpzoomOnboarding',
			array(
				'compatibilities' => $this->get_compatibilities(),
				'labels'          => $this->strings['localize_strings'],
			)
		);

		add_action(
			'admin_print_scripts',
			function() {
				echo '<script type="text/javascript">var wpzoom_ajax_url = \'' . esc_url( admin_url( 'admin-ajax.php' ) ) . '\';</script>';
			}
		);
	}

	/**
	 * Load WPZOOM Demo Importer class.
	 */
	public function setup_demo_import() {
		if ( zoom_current_theme_supports( 'wpz-onboarding', 'demo-importer' ) ) {
			require_once WPZOOM_INC . '/components/demo-importer/class-wpzoom-elementor-pages.php';
			require_once WPZOOM_INC . '/components/demo-importer/class-wpzoom-demo-import.php';
		}
	}

	/**
	 * Outputs the markup for the admin screen.
	 *
	 * @return void
	 */
	public function display_admin_page() {
		$license        = WPZOOM_Onboarding_License::get_instance();
		$class_prefix   = $this->classname_prefix;
		$theme_name     = $this->theme->get( 'Name' );
		$theme_version  = $this->theme->get( 'Version' );
		$step1_content  = $this->step_1_content();
		$step2_content  = $this->step_2_content();
		$step3_content  = $this->step_3_content();
		$post_url       = esc_url( admin_url( 'admin-post.php' ) );
		$license_key    = $license->get_license_key();
		$license_status = $license->get_license_status();
		$license_nonce  = wp_create_nonce( 'wpzoom_set_license' );
		/* translators: %s WPZOOM framework version */
		$framework_version = sprintf( __( 'ZOOM Framework %s', 'wpzoom' ), WPZOOM::$wpzoomVersion ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName.UsedPropertyNotSnakeCase

		extract( $this->strings ); // phpcs:ignore WordPress.PHP.DontExtract.extract_extract

		$license_status_badge = '';
		// Checks license status to display under license key.
		if ( ! $license_key ) {
			$label_license_note = $license->strings['enter-key'];
		} else {
			if ( ! get_transient( $this->theme_slug . '_license_message' ) ) {
				$license->check_license( $license_key );
			}
			$label_license_note = get_transient( $this->theme_slug . '_license_message' );
		}

		$label_badge               = '';
		$license_deactivate_button = '';
		$license_activate_button   = sprintf(
			'<input type="submit" value="%s" class="%s"/>',
			$label_license_submit,
			"{$class_prefix}content-main-license-submit button button-primary"
		);

		if ( 'valid' === $license_status ) {
			// Display "Deactivate" button.
			$license_deactivate_button = sprintf(
				'<input type="submit" value="%s" class="%s" data-deactivate-action="wpzoom_deactivate_license"/>',
				$this->strings['localize_strings']['deactivate_license'],
				"{$class_prefix}content-main-license-submit button button-secondary"
			);

			// Disable "Save & Activate" button.
			$license_activate_button = sprintf(
				'<input type="submit" value="%s" class="%s" disabled="disabled"/>',
				$label_license_submit,
				"{$class_prefix}content-main-license-submit button button-primary"
			);

			// Add label badge Active.
			$label_badge = '<span class="label-badge" data-license-status="' . esc_attr( $license_status ) . '">' . esc_html_x( 'Active', 'License status', 'wpzoom' ) . '</span>';
		} elseif ( ! empty( $license_key ) ) {
			$license_status_badge = '<span class="license-status-' . esc_attr( $license_status ) . '-badge"></span>';
			// Add label badge Inactive.
			$label_badge = '<span class="label-badge" data-license-status="' . esc_attr( $license_status ) . '">' . esc_html_x( 'Inactive', 'License status', 'wpzoom' ) . '</span>';
		}

		$snapshots_data         = '';
		$snapshots_data_content = '';
		if ( 1 === 2 ) { // TODO: At this moment we don't need to display Snapshots page. Change the false condition to apply_filters( 'wpzoom_demo_importer_debug_logs', false ).
			$snapshots_display_data = WPZOOM_Demo_Importer_Log::get_instance()->display_data();
			$snapshots_data         = "<li class=\"${class_prefix}tab {$class_prefix}tab-snapshots\"><a href=\"#snapshots\" title=\"Snapshots\">Snapshots</a></li>";
			$snapshots_data_content = "<div data-id=\"#snapshots\" class=\"{$class_prefix}content-main-tab {$class_prefix}content-main-snapshots\">
				<h3 class=\"{$class_prefix}content-main-title\">Snapshots</h3>
				<p class=\"{$class_prefix}content-main-intro\">Use snapshots to find out what changes a theme made to your database or to quickly restore the dev environment after testing database related changes.</p>
				<div class=\"{$class_prefix}content-log\">{$snapshots_display_data}</div>
			</div>";
		}

		$powered_by_elementor     = '';
		$is_elementor_recommended = WPZOOM_TGMPA_Assistance::get_instance()->is_recommended_plugin( 'elementor' );
		if ( $is_elementor_recommended ) {
			$powered_by_elementor = <<<POWERED_BY_ELEMENTOR
				<div class="{$class_prefix}content-side-section">
					<h3 class="{$class_prefix}content-side-section-title icon-elementor">
						<svg height="24" width="24" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">
							<defs>
								<linearGradient id="Gradient" x1="0.3508" y1="23.6642" x2="23.6642" y2="0.3508" gradientUnits="userSpaceOnUse">
									<stop offset="0" stop-color="#2e2975"/>
									<stop offset="1" stop-color="#f2295b"/>
								</linearGradient>
							</defs>
							<circle cx="12" cy="12" r="12" fill="url(#Gradient)"/>
							<rect height="9.5794" width="1.9158" x="7.2122" y="7.2122" fill="#ffffff"/>
							<rect height="1.9159" width="5.7525" x="11.0439" y="14.8757" fill="#ffffff"/>
							<rect height="1.9158" width="5.7525" x="11.0439" y="11.0439" fill="#ffffff"/>
							<rect height="1.9158" width="5.7525" x="11.0439" y="7.2074" fill="#ffffff"/>
						</svg>

						$label_elementor_title
					</h3>
					<p class="{$class_prefix}content-side-section-content">$label_elementor_content</p>
					<div class="{$class_prefix}content-side-section-button">
						<a href="https://www.wpzoom.com/go/elementor/" title="$label_elementor_button" target="_blank" class="button">$label_elementor_button</a>
					</div>
				</div>
POWERED_BY_ELEMENTOR;
		}

		// phpcs:ignore WordPress.Security.EscapeOutput
		echo <<<HERE
		<div class="{$class_prefix}wrapper">
			<div class="{$class_prefix}header">
				<div class="{$class_prefix}title-wrapper">
					<h1 class="{$class_prefix}title">$theme_name <small>$theme_version</small></h1>
					<h2 class="{$class_prefix}framework-version">$framework_version</h2>
				</div>

				<ul class="{$class_prefix}tabs">
					<li class="{$class_prefix}tab {$class_prefix}tab-quick-start active"><a href="#quick-start" title="$label_quickstart">$label_quickstart</a></li>
					<li class="{$class_prefix}tab {$class_prefix}tab-license"><a href="#license" title="$label_license">$label_license$license_status_badge</a></li>
					{$snapshots_data}
				</ul>
			</div>

			<div class="{$class_prefix}content-wrapper">
				<div class="{$class_prefix}content active">
					<div class="{$class_prefix}content-main">
						<div data-id="#quick-start" class="{$class_prefix}content-main-tab {$class_prefix}content-main-quickstart active">
							<h3 class="{$class_prefix}content-main-title">$label_content_title</h3>
							<p class="{$class_prefix}content-main-intro">$label_content_intro</p>

							<ol class="{$class_prefix}content-main-steps">

								<li id="step-choose-design" class="{$class_prefix}content-main-step step-1 step-choose-design">
									<h4 class="{$class_prefix}content-main-step-title">$label_steps_title2</h4>
									<p class="{$class_prefix}content-main-step-intro">$label_steps_intro2</p>
									<div class="{$class_prefix}content-main-step-content">$step2_content</div>
								</li>
	
								<li id="step-install-plugins" class="{$class_prefix}content-main-step step-2 step-install-plugins">
									<h4 class="{$class_prefix}content-main-step-title">$label_steps_title1</h4>
									<p class="{$class_prefix}content-main-step-intro">$label_steps_intro1</p>
									<div class="{$class_prefix}content-main-step-content">$step1_content</div>
								</li>



								<li id="step-import-demo" class="{$class_prefix}content-main-step step-3 step-import-demo">
									<h4 class="{$class_prefix}content-main-step-title">$label_steps_title3</h4>
									<p class="{$class_prefix}content-main-step-intro">$label_steps_intro3</p>
									<div class="{$class_prefix}content-main-step-content">$step3_content</div>
								</li>
							</ol>
						</div>

						<div data-id="#license" class="{$class_prefix}content-main-tab {$class_prefix}content-main-license">
							<h3 class="{$class_prefix}content-main-title">$label_license_title</h3>
							<p class="{$class_prefix}content-main-intro">$label_license_intro</p>

							<form method="post" action="{$post_url}">
								<fieldset>
									<input type="hidden" name="action" value="wpzoom_set_license" />
									<input type="hidden" name="wpzoom_set_license_nonce" value="{$license_nonce}" />

									<label for="wpzoom_license_key"><strong class="{$class_prefix}content-main-license-label">{$label_license_field_name}</strong>{$label_badge}</label>
									<input type="text" name="license_key" id="wpzoom_license_key" value="{$license_key}" class="{$class_prefix}content-main-license-key" data-license-status="{$license_status}" />

									<em class="{$class_prefix}content-main-license-note">{$label_license_note}</em>
									{$license_activate_button}{$license_deactivate_button}
								</fieldset>
							</form>
						</div>

						{$snapshots_data_content}
					</div>

					<div class="{$class_prefix}content-side">
						<div class="{$class_prefix}content-side-section">
							<h3 class="{$class_prefix}content-side-section-title icon-docs">
								<svg width="24" height="24" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">
									<path fill-rule="evenodd" clip-rule="evenodd" d="M7.96074 2H19.9019C20.9965 2 21.8921 2.9 21.8921 4V16C21.8921 17.1 20.9965 18 19.9019 18H7.96074C6.86614 18 5.97055 17.1 5.97055 16V4C5.97055 2.9 6.86614 2 7.96074 2ZM1.99017 6H3.98036V20H17.9117V22H3.98036C2.88576 22 1.99017 21.1 1.99017 20V6ZM19.9019 16H7.96075V4H19.9019V16ZM17.9117 9H9.95093V11H17.9117V9ZM9.95093 12H13.9313V14H9.95093V12ZM17.9117 6H9.95093V8H17.9117V6Z"/>
								</svg>

								$label_docs_title
							</h3>
							<p class="{$class_prefix}content-side-section-content">$label_docs_content</p>
							<div class="{$class_prefix}content-side-section-button">
								<a href="https://www.wpzoom.com/documentation/" title="$label_docs_button" target="_blank" class="button">$label_docs_button</a>
							</div>
						</div>

						{$powered_by_elementor}

						<div class="{$class_prefix}content-side-section">
							<h3 class="{$class_prefix}content-side-section-title icon-assist">
								<svg width="24" height="24" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">
									<path fill-rule="evenodd" clip-rule="evenodd" d="M15.9216 2H2.98533C2.43803 2 1.99023 2.45 1.99023 3V17L5.97062 13H15.9216C16.4689 13 16.9167 12.55 16.9167 12V3C16.9167 2.45 16.4689 2 15.9216 2ZM14.9265 4V11H5.14473L3.98047 12.17V4H14.9265ZM18.9068 6H20.897C21.4443 6 21.8921 6.45 21.8921 7V22L17.9117 18H6.96568C6.41837 18 5.97058 17.55 5.97058 17V15H18.9068V6Z"/>
								</svg>

								$label_assist_title
							</h3>
							<p class="{$class_prefix}content-side-section-content">$label_assist_content</p>
							<div class="{$class_prefix}content-side-section-button">
								<a href="https://wpzoom.com/support/tickets/" title="$label_assist_button" target="_blank" class="button">$label_assist_button</a>
							</div>
						</div>

						<div class="{$class_prefix}content-side-section">
							<h3 class="{$class_prefix}content-side-section-title icon-newwp">
								<svg width="24" height="25" viewBox="0 0 24 25" xmlns="http://www.w3.org/2000/svg">
									<path fill-rule="evenodd" clip-rule="evenodd" d="M11.5 19.6951V23.2951L12.93 22.6051C18.06 20.1351 21.5 15.1551 21.5 10.2051C21.5 4.96508 17.24 0.705078 12 0.705078C6.76 0.705078 2.5 4.96508 2.5 10.2051C2.5 15.2751 6.49 19.4351 11.5 19.6951ZM4.5 10.2051C4.5 6.06508 7.86 2.70508 12 2.70508C16.14 2.70508 19.5 6.06508 19.5 10.2051C19.5 13.9251 17.14 17.7051 13.5 20.0051V17.7051H12C7.86 17.7051 4.5 14.3451 4.5 10.2051ZM13 14.2051V16.2051H11V14.2051H13ZM12.7704 9.52675C11.9421 10.1617 11 10.8838 11 12.7051H13C13 11.6097 13.711 10.9883 14.4408 10.3506C15.21 9.6784 16 8.988 16 7.70508C16 5.49508 14.21 3.70508 12 3.70508C9.79 3.70508 8 5.49508 8 7.70508H10C10 6.60508 10.9 5.70508 12 5.70508C13.1 5.70508 14 6.60508 14 7.70508C14 8.58429 13.4202 9.02869 12.7704 9.52675Z"/>
								</svg>

								$label_newwp_title
							</h3>
							<p class="{$class_prefix}content-side-section-content">$label_newwp_content</p>
							<div class="{$class_prefix}content-side-section-button">
								<a href="https://wpzoom.com/video/" title="$label_newwp_button" class="button">$label_newwp_button</a>
							</div>
						</div>

						<div class="{$class_prefix}content-side-section">
							<h3 class="{$class_prefix}content-side-section-title icon-follow">
								<svg width="24" height="24" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">
									<path fill-rule="evenodd" clip-rule="evenodd" d="M20.8971 9H14.618L15.5633 4.43L15.5932 4.11C15.5932 3.7 15.424 3.32 15.1553 3.05L14.1005 2L7.55281 8.59C7.18462 8.95 6.9657 9.45 6.9657 10V20C6.9657 21.1 7.86129 22 8.95589 22H17.9118C18.7377 22 19.4442 21.5 19.7427 20.78L22.7479 13.73C22.8375 13.5 22.8872 13.26 22.8872 13V11C22.8872 9.9 21.9917 9 20.8971 9ZM20.897 13L17.9117 20H8.95587V10L13.2746 5.66003L12.17 11H20.897V13ZM4.9755 10H0.995117V22H4.9755V10Z"/>
								</svg>

								$label_follow_title
							</h3>
							<p class="{$class_prefix}content-side-section-content">$label_follow_content</p>
							<div class="{$class_prefix}content-side-section-button">
								<a href="https://twitter.com/wpzoom" target="_blank" title="$label_follow_button1" class="button button-smaller"><span class="dashicons dashicons-twitter"></span> <span class="icon-text">$label_follow_button1</span></a>
								<a href="https://facebook.com/wpzoom" target="_blank" title="$label_follow_button2" class="button button-smaller"><span class="dashicons dashicons-facebook-alt"></span> <span class="icon-text">$label_follow_button2</span></a>
								<a href="https://instagram.com/wpzoom" target="_blank" title="$label_follow_button3" class="button button-smaller"><span class="dashicons dashicons-instagram"></span> <span class="icon-text">$label_follow_button3</span></a>
							</div>
						</div>
					</div>
				</div>

				<div class="{$class_prefix}content-overlay">
					<div class="{$class_prefix}content-overlay-design-pages">
						<a href="#" class="go-back-link">
							<svg width="24" height="24" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">
								<path d="M19.9019 11H7.79167L13.3542 5.41L11.9412 4L3.98047 12L11.9412 20L13.3443 18.59L7.79167 13H19.9019V11Z"/>
							</svg>

							{$label_go_back}
						</a>

						<div class="{$class_prefix}content-container">
							<div class="{$class_prefix}content-overlay-design-pages-left-pane">
								<h5 class="{$class_prefix}content-overlay-design-pages-title"></h5>
								<a href="#" rel="noopener" class="{$class_prefix}content-overlay-design-import-demo-content button button-primary">{$label_import_button}</a>&nbsp;&nbsp;
								<a href="#" target="_blank" rel="noopener" class="{$class_prefix}content-overlay-design-pages-preview-link button button-secondary">{$label_live_preview}</a>

								<h6 class="{$class_prefix}content-overlay-design-pages-subhead">{$label_page_preview}</h6>
								<ul class="{$class_prefix}content-overlay-design-pages-thumbs"></ul>
							</div>

							<div class="{$class_prefix}content-overlay-design-pages-right-pane"><img src="" /></div>
						</div>
					</div>
				</div>
			</div>

			<div class="{$class_prefix}footer">
				<h3 class="{$class_prefix}footer-logo"><a href="https://wpzoom.com/" title="$label_wpzoom">$label_wpzoom</a></h3>

				<ul class="{$class_prefix}footer-links">
					<li class="{$class_prefix}footer-links-themes"><a href="https://www.wpzoom.com/themes/" target="_blank"  title="$label_themes">$label_themes</a></li>
                    <li class="{$class_prefix}footer-links-plugins"><a href="https://www.wpzoom.com/plugins/" target="_blank"  title="$label_plugins">$label_plugins</a></li>
					<li class="{$class_prefix}footer-links-blog"><a href="https://www.wpzoom.com/blog/" target="_blank" title="$label_blog">$label_blog</a></li>
					<li class="{$class_prefix}footer-links-support"><a href="https://www.wpzoom.com/support/" target="_blank" title="$label_support">$label_support</a></li>
				</ul>
			</div>
		</div>
HERE;
	}

	/**
	 * Returns the markup for Step 1 of the onboarding process.
	 *
	 * @return string
	 */
	public function step_1_content() {
		$plugins  = apply_filters( 'wpz_onboarding_sort_required_plugins', WPZOOM_TGMPA_Assistance::get_instance()->get_plugins() );
		$output   = '';
		$button   = '';
		$post_url = esc_url( admin_url( 'admin-post.php' ) );
		$nonce    = wp_create_nonce( 'wpzoom_required_plugins' );

		$disable_buttons = false;
		$notice_count    = $this->get_compatibilities( array( 'count' => true ) );

		// Disable buttons.
		if ( ! $notice_count['install_required'] &&
			! $notice_count['install_recommended'] &&
			! $notice_count['activate_recommended'] &&
			! $notice_count['activate_required'] ) {
			$disable_buttons = true;
		}

		if ( ! empty( $plugins ) ) {
			foreach ( $plugins as $plugin ) {

				$slug   = $plugin['slug'];
				$type   = $this->get_plugin_advise_type_text( $plugin['required'] );
				$status = $this->get_plugin_status_text( $slug );
				$data   = $this->get_plugin_data( $slug );

				$badge      = '';
				$classnames = array();

				if ( ! empty( $data ) ) {
					$name      = $data->name;
					$desc      = $data->short_description;
					$installed = $this->tgmpa_assistance->is_plugin_installed( $slug );
					$activated = $this->tgmpa_assistance->is_plugin_active( $slug );
					$disabled  = $activated ? 'disabled' : '';
					$plugin_data = '';

					$classnames[] = 'plugin_' . $slug;

					if( isset( $plugin['demos'] ) ) {
						foreach( $plugin['demos'] as $demo ) {
							$classnames[] =  'required-demo-' . $demo;
						}
						$plugin_data = 'data-plugin-sortable="true"';
					}

					if ( $installed ) {
						$classnames[] = 'plugin-status_' . preg_replace( '/\W+/', '-', strtolower( wp_strip_all_tags( $status ) ) );
						$badge       .= "<span class=\"plugin-badge\">{$status}</span>";
					}

					if ( ! $activated ) {
						$classnames[] = 'plugin-level_' . preg_replace( '/\W+/', '-', strtolower( wp_strip_all_tags( $type ) ) );
						$badge       .= "<span class=\"plugin-badge\">{$type}</span>";
					}

					// Prepare item class names.
					$class = implode( ' ', $classnames );

					$output .= <<<HERE
					<li class="{$class}" {$plugin_data}>
						<h5 class="plugin-name"><label><input type="checkbox" name="required_plugins[]" value="{$slug}" checked {$disabled} /> {$name}</label>{$badge}</h5>
						<p>{$desc}</p>
					</li>
HERE;
				}
			}

			$button  = '<input type="submit" name="button_submit" value="' . __( 'Install &amp; Activate', 'wpzoom' ) . '" class="button button-primary" ' . disabled( $disable_buttons, true, false ) . '/>';
			$button .= '<input type="button" name="button_checkall" value="' . __( 'Select all', 'wpzoom' ) . '" class="button button-secondary" style="margin-left:1em" ' . disabled( $disable_buttons, true, false ) . '/>';
		} else {
			$output .= '<li class="no-required-plugins"><h5><span class="dashicons dashicons-dismiss"></span> ' . __( 'There are no required plugins&hellip;', 'wpzoom' ) . '</h5></li>';
		}

		return <<<HERE
		<form method="post" action="{$post_url}">
			<fieldset>
				<input type="hidden" name="action" value="wpzoom_required_plugins" />
				<input type="hidden" name="wpzoom_required_plugins_nonce" value="{$nonce}" />

				<ul id="plugins-list" class="plugins-list">{$output}</ul>

				{$button}
			</fieldset>
		</form>
HERE;
	}

	/**
	 * Returns the markup for Step 2 of the onboarding process.
	 *
	 * @return string
	 */
	public function step_2_content() {
		$designs  = $this->get_theme_designs();
		$output   = '';
		$post_url = esc_url( admin_url( 'admin-post.php' ) );
		$nonce    = wp_create_nonce( 'wpzoom_theme_design' );

		extract( $this->strings ); // phpcs:ignore WordPress.PHP.DontExtract.extract_extract

		$attr_live_preview = esc_attr( $label_live_preview );
		$attr_view_pages   = esc_attr( $label_view_pages );

		if ( ! empty( $designs ) ) {
			$array_designs_keys = array_keys( $designs );
			$template_data      = $this->get_template_data();
			$imported_demo_id   = $this->get_imported_demo_id();

			foreach ( $designs as $id => $details ) {
				$selected_design = ( is_array( $template_data ) && isset( $template_data['name'] ) ? $template_data['name'] : $array_designs_keys[0] );
				$wxr_url         = ( is_array( $template_data ) && isset( $template_data['wxr_url'] ) ? $template_data['wxr_url'] : '' );
				$wxr_type        = ( is_array( $template_data ) && isset( $template_data['wxr_type'] ) ? $template_data['wxr_type'] : '' );
				$checked         = $id === $selected_design ? 'checked' : '';
				$name            = esc_html( $details['name'] );
				$name_attr       = esc_attr( $name );
				$thumbnail_url   = esc_url( $details['thumbnail'] );
				$preview_url     = esc_url( $details['preview_url'] );
				$classnames      = "design_{$id}";
				$design_id       = $this->get_design_id( $id );
				$delete_imported = '';

				if ( $id !== $selected_design ) {
					$label_selected = $this->strings['localize_strings']['select_template'];
				} elseif ( $design_id === $imported_demo_id ) {
					$classnames     .= ' selected-template imported-demo-content';
					$label_selected  = $this->strings['localize_strings']['imported'];
					$delete_imported = '<a href="#" class="delete-imported-demo-content" title="' . esc_html__( 'Delete imported demo content', 'wpzoom' ) . '" data-design="' . esc_attr( $id ) . '" data-wxr-url="' . esc_url( $wxr_url ) . '" data-wxr-type="' . esc_attr( $wxr_type ) . '"></a>';
				} else {
					$classnames    .= ' selected-template';
					$label_selected = $this->strings['label_selected'];
				}

				$output .= <<<HERE
				<li class="{$classnames}" data-design-id="{$design_id}">
					<figure title="{$name_attr}">
						<div class="preview-thumbnail" style="background-image:url('{$thumbnail_url}')"><button type="button" class="button-select-template">{$label_selected}</button></div>
						<figcaption>
							<h5>{$name}</h5>
							{$delete_imported}
							<p>
								<a href="{$preview_url}" target="_blank" rel="noopener" title="{$attr_live_preview}">{$label_live_preview}</a>
								<a href="#" title="{$attr_view_pages}" class="view-pages-open">{$label_view_pages}</a>
							</p>
							<input type="radio" name="theme_designs" value="{$id}" {$checked} />
						</figcaption>
					</figure>
				</li>
HERE;
			}
		} else {
			$output .= '<li class="no-theme-designs"><h5><span class="dashicons dashicons-dismiss"></span> ' . __( 'There are no theme designs&hellip;', 'wpzoom' ) . '</h5></li>';
		}

		return <<<HERE
		<form method="post" action="{$post_url}">
			<fieldset>
				<input type="hidden" name="action" value="wpzoom_theme_design" />
				<input type="hidden" name="wpzoom_theme_design_nonce" value="{$nonce}" />

				<ul>{$output}</ul>
			</fieldset>
		</form>
HERE;
	}

	/**
	 * Returns the markup for Step 3 of the onboarding process.
	 *
	 * @return string
	 */
	public function step_3_content() {
		$output              = '';
		$class_prefix        = $this->classname_prefix;
		$designs             = $this->get_theme_designs();
		$label_template      = $this->strings['label_template'];
		$label_import_button = $this->strings['label_import_button'];

		extract( $this->strings ); // phpcs:ignore WordPress.PHP.DontExtract.extract_extract

		$import_button_disabled = '';
		$array_designs_keys     = array_keys( $designs );
		$template               = option::get( 'layout-site' ) ?: $array_designs_keys[0]; // phpcs:ignore WordPress.PHP.DisallowShortTernary
		$template_name          = ( isset( $designs[ $template ] ) ? esc_html( $designs[ $template ]['name'] ) : $template );
		$compatibilities        = $this->get_compatibilities();
		$compatibility_notice   = '';
		$notice_items           = '';
		$notice_actions         = array();
		$recommended_notice     = 0;
		$template_data          = $this->get_template_data();

		$selected_design = ( is_array( $template_data ) && isset( $template_data['name'] ) ? $template_data['name'] : $template );
		$wxr_type        = ( is_array( $template_data ) && isset( $template_data['wxr_type'] ) ? $template_data['wxr_type'] : 'invalid' );
		$wxr_url         = ( is_array( $template_data ) && isset( $template_data['wxr_url'] ) ? $template_data['wxr_url'] : '' );

		foreach ( $compatibilities as $type => $compatibility ) {
			if ( ! empty( $compatibility ) ) {
				if ( 'errors' === $type ) {
					$status = __( 'Required', 'wpzoom' );
					$level  = 'required';
				} else {
					$status             = __( 'Recommended', 'wpzoom' );
					$level              = 'recommended';
					$recommended_notice = count( $compatibility );
				}
				if ( empty( $compatibility_notice ) ) {
					$compatibility_notice .= '<ul>##LIST##</ul>';
				}
				foreach ( $compatibility as $prop => $data ) {
					$notice_item  = '<h3>' . $data['title'] . '<span class="plugin-badge">' . $status . '</span></h3>';
					$notice_item .= $data['description'];

					$notice_items .= WPZOOM_Onboarding_Utils::wrap_in_li(
						$notice_item,
						array(
							'class'       => "plugin-level_{$level}",
							'data-notice' => $prop,
						)
					);
				}
			}
		}

		if ( $recommended_notice > 0 ) {
			$link_template                      = '<a id="wpz-onboard-skip-notice" href="#">%s</a>';
			$notice_actions['skip-recommended'] = sprintf(
				$link_template,
				translate_nooped_plural( $this->strings['notice_skip_recommended_link'], $recommended_notice, 'wpzoom' )
			);
		}

		if ( ! empty( $compatibility_notice ) ) {
			$import_button_disabled = 'disabled';
			$compatibility_notice   = str_replace( '##LIST##', $notice_items, $compatibility_notice );
			$compatibility_notice  .= $notice_actions['skip-recommended'];

			$output .= <<<HERE
			<div class="{$class_prefix}notice">
				{$compatibility_notice}
			</div>
HERE;
		}

		if ( ! $wxr_url ) {
			$import_button_disabled = 'disabled';
			$label_import_button    = esc_html__( 'Invalid .xml URL provided', 'wpzoom' );
		}

		$output .= <<<HERE
		<h3 class="{$class_prefix}selected-template">{$label_template} <strong>{$template_name}</strong></h3>
		<p class="{$class_prefix}import-button-wrap">
			<a href="#" class="button button-primary {$import_button_disabled} {$class_prefix}import-button">{$label_import_button}</a>
			<input type="hidden" name="wpzoom-selected-design" value="{$selected_design}"/>
			<input type="hidden" name="wpzoom-wxr-url" value="{$wxr_url}"/>
			<input type="hidden" name="wpzoom-wxr-type" value="{$wxr_type}"/>
		</p>
HERE;

		return $output;
	}

	/**
	 * Import Compatibility Errors
	 *
	 * @return mixed
	 */
	public function get_compatibilities_data() {
		return array(
			'xmlreader'                       => array(
				'title'       => esc_html__( 'XMLReader Support Missing', 'wpzoom' ),
				/* translators: %s doc link. */
				'description' => '<p>' . esc_html__( "You're close to importing the template. To complete the process, enable XMLReader support on your website.", 'wpzoom' ) . '</p><p>' . esc_html__( 'Just get in touch with your service administrator and request them to enable XMLReader on your website. Once this is done you can try importing template again.', 'wpzoom' ) . '</p>',
			),
			'child-theme'                     => array(
				'title'       => esc_html__( 'Child Theme is active', 'wpzoom' ),
				/* translators: %s Parent theme name. */
				'description' => '<p>' . sprintf( esc_html__( 'Please switch to the parent theme %s to import the demo content.', 'wpzoom' ), $this->theme->get( 'Name' ) ) . '</p><p>' . esc_html__( 'The importer works only the parent theme is active. It doesn’t support Child Themes or themes which folders were renamed to something different.', 'wpzoom' ) . '</p>',
			),
			'wp-debug'                        => array(
				'title'       => esc_html__( 'Disable Debug Mode', 'wpzoom' ),
				/* translators: %s doc link. */
				'description' => '<p>' . esc_html__( 'WordPress debug mode is currently enabled on your website. With this, any errors from third-party plugins might affect the import process.', 'wpzoom' ) . '</p><p>' . esc_html__( 'Please disable it to continue importing demo content. To do so, you can add the following code into the wp-config.php file.', 'wpzoom' ) . '</p><p><code>define(\'WP_DEBUG\', false);</code></p>',
			),
			'image-resizing-not-supported'    => array(
				'title'       => esc_html__( "Image Editor doesn't support image resizing", 'wpzoom' ),
				'description' => esc_html__( "The regenerate attachments tool won't be able to do anything because your server doesn't support image editing which means that WordPress can't create thumbnail images. Please ask your host to install the Imagick or GD PHP extensions.", 'wpzoom' ),
			),
			'notice_ask_to_update_maybe'      => array(
				'title'       => '', // retrieve title from $this->srtings.
				/* translators: %s update page link. */
				'description' => '<p>##LIST##<p>' . sprintf( __( 'Please <a href="%s" target="_blank">update</a> them for a successful import.', 'wpzoom' ), esc_url( network_admin_url( 'update-core.php' ) ) ) . '</p>',
			),
			'notice_ask_to_update'            => array(
				'title'       => '', // retrieve title from $this->srtings.
				/* translators: %s update page link. */
				'description' => '<p>##LIST##<p>' . sprintf( __( 'Please <a href="%s" target="_blank">update</a> them for a successful import. Skipping this step might break the template design/feature.', 'wpzoom' ), esc_url( network_admin_url( 'update-core.php' ) ) ) . '</p>',
			),
			'notice_can_install_required'     => array(
				'title'       => '', // retrieve title from $this->srtings.
				/* translators: %s required plugins. */
				'description' => '<p><strong>' . esc_html__( 'IMPORTANT', 'wpzoom' ) . ':</strong> ##LIST##</p>',
			),
			'notice_can_activate_required'    => array(
				'title'       => '', // retrieve title from $this->srtings.
				/* translators: %s required plugins. */
				'description' => '<p>##LIST##</p>',
			),
			'notice_can_install_recommended'  => array(
				'title'       => '', // retrieve title from $this->srtings.
				/* translators: %s required plugins. */
				'description' => '<p>##LIST##</p>',
			),
			'notice_can_activate_recommended' => array(
				'title'       => '', // retrieve title from $this->srtings.
				'description' => '<p>##LIST##</p>',
			),
			'empty_license_key'               => array(
				'title'       => esc_html__( 'Activate Your License Key', 'wpzoom' ),
				/* translators: %1$s License tab url. %2$s WPZOOM Members Area -> Licenses */
				'description' => '<p>' . sprintf( __( 'Please %1$s and activate your license key to import the demo content. You can find your license in the <a href="%2$s" target="_blank">Members Area → License Keys</a>.', 'wpzoom' ), '<a href="#license" id="license-tab-link">' . esc_html_x( 'enter', 'License text link', 'wpzoom' ) . '</a>', esc_url( 'https://www.wpzoom.com/account/licenses/' ) ) . '</p>',
			),
			'inactive_license_key'            => array(
				'title'       => esc_html__( 'Inactive License Key', 'wpzoom' ),
				/* translators: %s License tab url. */
				'description' => '<p>' . sprintf( __( 'Please %s your license key to import the demo content.', 'wpzoom' ), '<a href="#license" id="license-tab-link">' . esc_html_x( 'activate', 'License text link', 'wpzoom' ) . '</a>' ) . '</p>',
			),
			'expired_license_key'             => array(
				'title'       => esc_html__( 'Expired License Key', 'wpzoom' ),
				/* translators: %1$s License tab url. %2$s Renew license url. */
				'description' => '<p>' . sprintf( __( "Your license key has been expired and now you can't import the demo content. Please %1\$s or %2\$s your license key to be able to import the demo content.", 'wpzoom' ), '<a href="#license" id="license-tab-link">' . esc_html_x( 'activate', 'License text link', 'wpzoom' ) . '</a>', '<a href="' . esc_url( WPZOOM_Onboarding_License::get_instance()->get_renewal_link() ) . '" target="_blank">' . esc_html_x( 'renew', 'License renew link', 'wpzoom' ) . '</a>' ) . '</p>',
			),
		);
	}

	/**
	 * Get all compatibilities
	 *
	 * @param array $args Array of arguments to get.
	 * @return array
	 */
	public function get_compatibilities( $args = array() ) {
		// Run tgmpa register action when doing ajax request.
		if ( wp_doing_ajax() ) {
			do_action( 'tgmpa_register' );
		}

		$return  = array();
		$data    = $this->get_compatibilities_data();
		$plugins = apply_filters( 'wpz_onboarding_sort_required_plugins', WPZOOM_TGMPA_Assistance::get_instance()->get_plugins() );

		$compatibilities = array(
			'errors'   => array(),
			'warnings' => array(),
		);
		$message         = array();
		$count           = array(
			'plugin_group'         => 0,
			'install_required'     => 0,
			'install_recommended'  => 0,
			'update'               => 0,
			'update_maybe'         => 0,
			'activate_recommended' => 0,
			'activate_required'    => 0,
		);

		$license_key    = WPZOOM_Onboarding_License::get_instance()->get_license_key();
		$license_status = WPZOOM_Onboarding_License::get_instance()->get_license_status();
		if ( ! $license_key ) {
			$compatibilities['errors']['empty_license_key'] = $data['empty_license_key'];
		} else {
			if ( 'invalid' === $license_status || 'site_inactive' === $license_status || 'inactive' === $license_status ) {
				$compatibilities['errors']['inactive_license_key'] = $data['inactive_license_key'];
			} elseif ( 'expired' === $license_status ) {
				$compatibilities['errors']['expired_license_key'] = $data['expired_license_key'];
			} elseif ( 'valid' !== $license_status ) {
				$compatibilities['errors']['inactive_license_key'] = $data['inactive_license_key'];
			}
		}

		if ( ! empty( $plugins ) ) {
			foreach ( $plugins as $slug => $plugin ) {
				if ( $this->tgmpa_assistance->is_plugin_active( $slug ) && false === $this->tgmpa_assistance->does_plugin_have_update( $slug ) ) {
					continue;
				}

				if ( ! $this->tgmpa_assistance->is_plugin_installed( $slug ) ) {
					if ( current_user_can( 'install_plugins' ) ) {
						if ( true === $plugin['required'] ) {
							$count['install_required']               += 1;
							$message['notice_can_install_required'][] = $slug;
						} else {
							$count['install_recommended']               += 1;
							$message['notice_can_install_recommended'][] = $slug;
						}
					}
				} else {
					if ( ! $this->tgmpa_assistance->is_plugin_active( $slug ) && $this->tgmpa_assistance->can_plugin_activate( $slug ) ) {
						if ( current_user_can( 'activate_plugins' ) ) {
							if ( true === $plugin['required'] ) {
								$count['activate_required']               += 1;
								$message['notice_can_activate_required'][] = $slug;
							} else {
								$count['activate_recommended']               += 1;
								$message['notice_can_activate_recommended'][] = $slug;
							}
						}
					}

					if ( $this->tgmpa_assistance->does_plugin_require_update( $slug ) || false !== $this->tgmpa_assistance->does_plugin_have_update( $slug ) ) {
						if ( current_user_can( 'update_plugins' ) ) {
							if ( $this->tgmpa_assistance->does_plugin_require_update( $slug ) ) {
								$count['update']                  += 1;
								$message['notice_ask_to_update'][] = $slug;
							} elseif ( false !== $this->tgmpa_assistance->does_plugin_have_update( $slug ) ) {
								$count['update_maybe']                  += 1;
								$message['notice_ask_to_update_maybe'][] = $slug;
							}
						}
					}
				}
			}
			unset( $slug, $plugin );

			// If we have notices to display, we move forward.
			if ( ! empty( $message ) ) {
				krsort( $message ); // Sort messages.
				$rendered       = '';
				$rendered_title = '';

				if ( ! current_user_can( 'activate_plugins' ) && ! current_user_can( 'install_plugins' ) && ! current_user_can( 'update_plugins' ) ) {
					$rendered = esc_html( $this->strings['notice_cannot_install_activate'] ) . ' ' . esc_html( $this->strings['contact_admin'] );
				} else {
					// Render the individual message lines for the notice.
					foreach ( $message as $type => $plugin_group ) {

						if ( ! isset( $data[ $type ] ) ) {
							continue;
						}
						$list_plugins = array();

						// Get the external info link for a plugin if one is available.
						foreach ( $plugin_group as $plugin_slug ) {
							$plugin_data = $this->get_plugin_data( $plugin_slug );
							if ( is_object( $plugin_data ) && isset( $plugin_data->name ) ) {
								$list_plugins[] = WPZOOM_Onboarding_Utils::wrap_in_li( $plugin_data->name, array( 'data-plugin-slug' => $plugin_slug ) );
							}
						}
						unset( $plugin_slug );

						$count['plugin_group'] = count( $plugin_group );
						$list                  = ! empty( $list_plugins ) ? WPZOOM_Onboarding_Utils::wrap_in_ul( implode( '', $list_plugins ) ) : '';

						$rendered = sprintf(
							translate_nooped_plural( $this->strings[ $type ], $count['plugin_group'], 'wpzoom' ),
							$list,
							$count['plugin_group']
						);

						$count_type = '';
						$title_type = '';

						if ( false !== strpos( $type, 'notice_can_' ) ) {
							$count_type = str_replace( 'notice_can_', '', $type );
							$title_type = str_replace( 'notice_can', 'title', $type );
						} elseif ( false !== strpos( $type, 'notice_ask_to_' ) ) {
							$count_type = str_replace( 'notice_ask_to_', '', $type );
							$title_type = str_replace( 'notice_ask_to', 'title', $type );
						}

						if ( $title_type && isset( $this->strings[ $title_type ] ) && isset( $count[ $count_type ] ) ) {
							$rendered_title = translate_nooped_plural( $this->strings[ $title_type ], $count[ $count_type ], 'wpzoom' );
						}
						if ( ! empty( $rendered_title ) ) {
							$data[ $type ]['title'] = $rendered_title;
						}
						$data[ $type ]['description'] = str_replace( '##LIST##', $rendered, $data[ $type ]['description'] );

						if ( 'notice_can_install_required' === $type || 'notice_can_activate_required' == $type ) {
							$compatibilities['errors'][ $type ] = $data[ $type ];
						} else {
							$compatibilities['warnings'][ $type ] = $data[ $type ];
						}
					}
					unset( $type, $plugin_group, $list_plugins, $list );
				}
			}
		}

		if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {
			$compatibilities['warnings']['wp-debug'] = $data['wp-debug'];
		}

		if ( ! wp_image_editor_supports( array( 'methods' => array( 'resize' ) ) ) ) {
			$compatibilities['warning']['image-resizing-not-supported'] = $data['image-resizing-not-supported'];
		}

		if ( ! class_exists( 'XMLReader' ) ) {
			$compatibilities['errors']['xmlreader'] = $data['xmlreader'];
		}

		if ( is_child_theme() ) {
			$compatibilities['errors']['child-theme'] = $data['child-theme'];
		}

		$return = $compatibilities;
		if ( isset( $args['errors'] ) ) {
			$return = $compatibilities['errors'];
		} elseif ( isset( $args['warnings'] ) ) {
			$return = $compatibilities['warnings'];
		} elseif ( isset( $args['count'] ) ) {
			$return = $count;
		} elseif ( isset( $args['message'] ) ) {
			$return = $message;
		}
		return $return;
	}

	/**
	 * Handles AJAX requests to get all installed plugins.
	 *
	 * @return void
	 */
	public function get_installed_plugins_ajax() {
		// phpcs:ignore
		if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'wpzoom_required_plugins' ) || ! current_user_can( 'install_plugins' ) ) {
			wp_send_json_error();
		}

		$installed_plugins = preg_replace( array( '/[\\/].+$/i', '/\.php$/i' ), '', array_keys( get_plugins() ) );
		wp_send_json_success( $installed_plugins );
	}

	/**
	 * Handles AJAX requests to install plugins.
	 *
	 * @return void
	 */
	public function plugin_install_ajax() {
		global $wp_filesystem;

		// phpcs:ignore
		if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'wpzoom_required_plugins' ) || ! current_user_can( 'install_plugins' ) || ! isset( $_POST['plugins'] ) || empty( $_POST['plugins'] ) ) {
			wp_send_json_error();
		}

		$plugins = array_filter( array_map( 'sanitize_key', (array) $_POST['plugins'] ) );

		require_once ABSPATH . 'wp-admin/includes/file.php';
		require_once ABSPATH . 'wp-admin/includes/plugin.php';
		require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
		require_once ABSPATH . 'wp-admin/includes/plugin-install.php';

		$skin     = new WP_Ajax_Upgrader_Skin();
		$upgrader = new Plugin_Upgrader( $skin );

		do_action( 'tgmpa_register' );

		$plugins_to_activate = array();
		$json_data           = array(
			'success' => array(),
			'error'   => array(),
		);

		// Grab the file paths for the selected & inactive plugins from the registration array.
		foreach ( $plugins as $slug ) {
			if ( $this->tgmpa_assistance->is_plugin_installed( $slug ) ) {
				if ( $this->tgmpa_assistance->can_plugin_activate( $slug ) ) {
					$plugins_to_activate[] = $this->tgmpa_assistance->plugins[ $slug ]['file_path'];

					$json_data['success'][ $slug ] = array(
						'type'     => 'activate',
						'filePath' => $this->tgmpa_assistance->plugins[ $slug ]['file_path'],
					);
				}
			} else {
				$plugin_data = $this->get_plugin_data( $slug, array( 'clear_update_cache' => true ) );
				$source      = is_object( $plugin_data ) && isset( $plugin_data->download_link ) ? $plugin_data->download_link : $this->tgmpa_assistance->get_download_url( $slug );

				if ( $source ) {

					$result = $upgrader->install( $source );

					if ( is_wp_error( $result ) ) {
						$result->add_data( array( 'status' => 500 ) );

						return $result;
					}

					// This should be the same as $result above.
					if ( is_wp_error( $skin->result ) ) {
						$skin->result->add_data( array( 'status' => 500 ) );

						return $skin->result;
					}

					if ( $skin->get_errors()->has_errors() ) {
						$error = $skin->get_errors();
						$error->add_data( array( 'status' => 500 ) );

						return $error;
					}

					if ( is_null( $result ) ) {
						// Pass through the error from WP_Filesystem if one was raised.
						if ( $wp_filesystem instanceof WP_Filesystem_Base
							&& is_wp_error( $wp_filesystem->errors ) && $wp_filesystem->errors->has_errors()
						) {
							return new WP_Error(
								'unable_to_connect_to_filesystem',
								$wp_filesystem->errors->get_error_message(),
								array( 'status' => 500 )
							);
						}

						return new WP_Error(
							'unable_to_connect_to_filesystem',
							__( 'Unable to connect to the filesystem. Please confirm your credentials.', 'wpzoom' ),
							array( 'status' => 500 )
						);
					}

					$file_path = $upgrader->plugin_info(); // Grab the plugin info from the Plugin_Upgrader method.

					if ( ! $file_path ) {
						return new WP_Error(
							'unable_to_determine_installed_plugin',
							__( 'Unable to determine what plugin was installed.', 'wpzoom' ),
							array( 'status' => 500 )
						);
					}

					if ( $this->tgmpa_assistance->can_plugin_activate( $slug ) ) {
						$plugins_to_activate[] = $file_path;

						$json_data['success'][ $slug ] = array(
							'type'     => 'activate',
							'filePath' => $file_path,
						);
					}
				} else {
					$json_data['error'][ $slug ] = array(
						'type'    => 'invalid_download_link',
						'message' => __( 'Invalid download link.', 'wpzoom' ),
					);
				}
			}

			$json_data['success'][ $slug ]['required'] = $this->tgmpa_assistance->plugins[ $slug ]['required'];

			if ( isset( $json_data['error'][ $slug ] ) ) {
				$json_data['error'][ $slug ]['required'] = $this->tgmpa_assistance->plugins[ $slug ]['required'];
			}
		}
		unset( $slug );

		// Return early if there are no plugins to activate.
		if ( empty( $plugins_to_activate ) ) {
			wp_send_json_error( esc_html__( 'No plugins are available to be activated at this time.', 'wpzoom' ) );
		}

		// Now we are good to go - let's start activating plugins.
		$activate = activate_plugins( $plugins_to_activate );

		if ( is_wp_error( $activate ) ) {
			$error_data = $activate->get_error_data();
			foreach ( $error_data as $plugin => $data ) {
				$slug = preg_replace( array( '/[\\/].+$/i', '/\.php$/i' ), '', $plugin );

				$json_data['error'][ $slug ]['message'] = wp_kses_post( $data->get_error_message() );
			}
		}

		wp_send_json_success( $json_data );
	}

	/**
	 * Handles AJAX requests to get the details of a given theme design.
	 *
	 * @return void
	 */
	public function get_theme_design_ajax() {
		// phpcs:ignore
		if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'wpzoom_theme_design' ) || ! current_user_can( 'manage_options' ) || ! isset( $_POST['design'] ) || empty( $_POST['design'] ) ) {
			wp_send_json_error();
		}

		$design  = sanitize_key( $_POST['design'] );
		$designs = $this->get_theme_designs();

		if ( array_key_exists( $design, $designs ) ) {
			wp_send_json_success( $designs[ $design ] );
		} else {
			wp_send_json_error();
		}
	}

	/**
	 * Receive plugin data by plugin slug.
	 *
	 * @param string $plugin_slug Plugin slug to get data for.
	 * @param array  $args Other arguments for get a plugin data.
	 * @return object|array The plugin data object on success (name, short_description, version, author). Otherwise empty array.
	 */
	public function get_plugin_data( $plugin_slug, $args = array() ) {
		$defaults    = array(
			'clear_update_cache' => false,
		);
		$parsed_args = wp_parse_args( $args, $defaults );

		if ( $parsed_args['clear_update_cache'] ) {
			// Clear cache so we knows about the new plugin.
			$this->clear_plugins_cache();
		}

		$data               = new stdClass();
		$installed_plugins  = $this->tgmpa_assistance->get_plugins();
		$cache_plugins_data = WPZOOM_Onboarding_Utils::onboarding_cache_get( 'wpz_onboarding', 'plugins_data' );

		if ( ! $cache_plugins_data ) {
			$cache_plugins_data = array();
		}

		foreach ( $this->tgmpa_assistance->plugins as $slug => $plugin ) {
			if ( $plugin_slug === $slug ) {
				if ( isset( $cache_plugins_data[ $slug ] ) ) { // Get cached plugins data to prevent calling function plugins_api() on each page load.
					$data = $cache_plugins_data[ $slug ];
				} elseif ( isset( $installed_plugins[ $plugin['file_path'] ] ) ) { // Get plugin data only for installed plugins.
					$plugin_data = $installed_plugins[ $plugin['file_path'] ];

					// Store plugin data object.
					$data->name              = $plugin_data['Name'];
					$data->slug              = $slug;
					$data->short_description = $plugin_data['Description'];
					$data->version           = $plugin_data['Version'];
					$data->author            = sprintf( '<a href="%s">%s</a>', $plugin_data['AuthorURI'], $plugin_data['Author'] );
					$data->download_link     = '';

					$cache_plugins_data[ $slug ] = $data;
				} else { // Get plugin data for not installed yet plugins.
					if ( ! function_exists( 'plugins_api' ) ) {
						require_once ABSPATH . 'wp-admin/includes/plugin-install.php';
					}
					$api_data = plugins_api(
						'plugin_information',
						array(
							'slug'   => $slug,
							'fields' => array(
								'short_description' => true,
								'sections'          => false,
							),
						)
					);

					if ( ! is_wp_error( $api_data ) ) {
						$data->name              = $plugin['name'];
						$data->slug              = $api_data->slug;
						$data->short_description = $api_data->short_description;
						$data->version           = $api_data->version;
						$data->author            = $api_data->author;
						$data->download_link     = $api_data->download_link;

						// Cache plugin data.
						$cache_plugins_data[ $slug ] = $data;
					} elseif ( 'plugins_api_failed' === $api_data->get_error_code() ) { // Plugin not found.
						$data->name              = $plugin['name'];
						$data->slug              = $slug;
						$data->short_description = '';

						// Cache plugin data.
						$cache_plugins_data[ $slug ] = $data;
					}
				}
			}
		}

		// Update cached plugins data.
		if ( $parsed_args['clear_update_cache'] ) {
			WPZOOM_Onboarding_Utils::onboarding_cache_set( 'wpz_onboarding', $cache_plugins_data, 'plugins_data' );
		}

		// Check if empty data.
		$tmp = (array) $data;
		if ( empty( $tmp ) ) {
			return array();
		}

		return $data;
	}

	/**
	 * Sort plugins by Required/Recommended type and by alphabetical plugin name within each type.
	 *
	 * @param array $items Plugins array.
	 * @return array Sorted plugins items.
	 */
	public function sort_plugins( $items ) {
		$type = array();
		$name = array();

		foreach ( $items as $i => $plugin ) {
			$type[ $i ] = $plugin['required'];
			$name[ $i ] = $plugin['name'];
		}

		array_multisort( $type, SORT_DESC, $name, SORT_ASC, $items );

		return $items;
	}

	/**
	 * Receive theme designs registered to the filter 'wpz_onboarding_theme_designs'
	 *
	 * @param array $designs Custom theme designs.
	 * @return array Filtered theme designs.
	 */
	public function get_theme_designs( $designs = array() ) {
		$designs = array_filter( apply_filters( 'wpz_onboarding_theme_designs', $designs ) );
		foreach ( $designs as $design => $props ) {
			if ( isset( $props['preview_pages'] ) && is_array( $props['preview_pages'] ) ) {
				foreach ( $props['preview_pages'] as $key => $page ) {
					/* translators: %1$s Page image src, %2$s Page title */
					$designs[ $design ]['preview_pages'][ $key ]['preview_img'] = sprintf( '<img src="%1$s" alt="%2$s"/>', esc_url( $page['fullsize'] ), esc_attr( $page['name'] ) );
				}
			}
		}
		return $designs;
	}

	/**
	 * Get template data
	 *
	 * @param string $template_name Template name.
	 * @return mixed The template data if exists, otherwise false.
	 */
	public function get_template_data( $template_name = '' ) {
		$cache_template_data = WPZOOM_Onboarding_Utils::onboarding_cache_get( 'wpz_demo_import', 'template_data' );

		if ( ! $cache_template_data || ( is_array( $cache_template_data ) && ! $cache_template_data['wxr_url'] ) ) {
			// Set default design if template data cache is empty.
			$this->set_selected_design();

			// Overwrite cached value.
			$cache_template_data = WPZOOM_Onboarding_Utils::onboarding_cache_get( 'wpz_demo_import', 'template_data' );
		}

		if ( '' !== $template_name && $cache_template_data['name'] !== $template_name ) {
			return false;
		}

		return $cache_template_data;
	}

	/**
	 * Get selected design
	 *
	 * @return string
	 */
	public function get_selected_design() {
		$cache_template_data = WPZOOM_Onboarding_Utils::onboarding_cache_get( 'wpz_demo_import', 'template_data' );
		$designs             = $this->get_theme_designs();
		$array_designs_keys  = array_keys( $designs );
		$selected_design     = get_theme_mod( 'layout-site', $array_designs_keys[0] );

		if ( is_array( $cache_template_data ) ) {
			return $cache_template_data['name'];
		} else {
			return $selected_design;
		}
	}

	/**
	 * Set selected design
	 *
	 * @param string $design The design slug to set as selected.
	 * @return mixed The passed design if is not present in the designs array, otherwise return void.
	 */
	public function set_selected_design( $design = '' ) {
		$designs            = $this->get_theme_designs();
		$array_designs_keys = array_keys( $designs );

		if ( empty( $design ) && isset( $array_designs_keys[0] ) ) {
			$design = option::get( 'layout-site' ) ?: $array_designs_keys[0]; // phpcs:ignore WordPress.PHP.DisallowShortTernary
		}
		if ( ! array_key_exists( $design, $designs ) ) {
			return $design;
		}

		$cache_template_data = WPZOOM_Onboarding_Utils::onboarding_cache_get( 'wpz_demo_import', 'template_data' );

		if ( ! $cache_template_data || ( is_array( $cache_template_data ) && ( $cache_template_data['name'] !== $design || ! $cache_template_data['wxr_url'] ) ) ) {
			option::set( 'layout-site', $design );
			set_theme_mod( 'wpz_multiple_demo_importer', $design );

			$design_id        = $this->get_design_id( $design );
			$imported_demo_id = $this->get_imported_demo_id();
			$xml_data         = get_demo_xml_data();

			if ( $xml_data['local']['response'] ) {
				$wxr_url  = esc_url( $xml_data['local']['url'] );
				$wxr_type = 'local';
			} elseif ( $xml_data['remote']['response'] ) {
				$wxr_url  = esc_url( $xml_data['remote']['url'] );
				$wxr_type = 'remote';
			} else {
				$wxr_url  = '';
				$wxr_type = 'invalid';
			}

			if ( is_child_theme() ) {
				$wxr_type = 'invalid';
				$wxr_url  = '';
			}

			$data = array(
				'name'     => $design,
				'wxr_type' => $wxr_type,
				'wxr_url'  => $wxr_url,
				'imported' => ( $design_id === $imported_demo_id ),
			);

			WPZOOM_Onboarding_Utils::onboarding_cache_set( 'wpz_demo_import', $data, 'template_data' );
		}
	}

	/**
	 * Get the plugin required/recommended text string.
	 *
	 * @param string $required Plugin required setting.
	 * @return string
	 */
	protected function get_plugin_advise_type_text( $required ) {
		if ( true === $required ) {
			return __( 'Required', 'wpzoom' );
		}

		return __( 'Recommended', 'wpzoom' );
	}

	/**
	 * Determine the plugin status message.
	 *
	 * @param string $slug Plugin slug.
	 * @return string
	 */
	protected function get_plugin_status_text( $slug ) {
		if ( ! $this->tgmpa_assistance->is_plugin_installed( $slug ) ) {
			return __( 'Not Installed', 'wpzoom' );
		}

		if ( ! $this->tgmpa_assistance->is_plugin_active( $slug ) ) {
			$install_status = __( 'Installed, Not Activated', 'wpzoom' );
		} else {
			$install_status = __( 'Active', 'wpzoom' );
		}

		return $install_status;
	}

	/**
	 * Clears the plugins cache used by WPZOOM_Onboarding::get_plugin_data().
	 *
	 * @return void
	 */
	public function clear_plugins_cache() {
		WPZOOM_Onboarding_Utils::onboarding_cache_delete( 'wpz_onboarding', 'plugins_data' );
	}

	/**
	 * Get imported demo id
	 *
	 * @return mixed Imported demo id or empty value if no one demo was imported, otherwise return false if the index not found in the demos data array.
	 */
	public function get_imported_demo_id() {
		$theme_demos_data = get_demos_details();
		if ( isset( $theme_demos_data['imported'] ) ) {
			return $theme_demos_data['imported'];
		}
		return false;
	}

	/**
	 * Get design id by provided design slug
	 *
	 * @param string $design_slug The registered design slug.
	 * @return string
	 */
	public function get_design_id( $design_slug ) {
		$design_id = $design_slug;

		// Get design id only if theme supports multiple designs.
		if ( current_theme_supports( 'wpz-multiple-demo-importer' ) ) {
			$theme_demos_data = get_demos_details();
			$demo_key         = array_search( $design_slug, array_column( $theme_demos_data['demos'], 'name' ) ); // phpcs:ignore PHPCompatibility.FunctionUse.NewFunctions.array_columnFound
			$design_id        = isset( $theme_demos_data['demos'][ $demo_key ] ) ? $theme_demos_data['demos'][ $demo_key ]['id'] : '';
		}
		return $design_id;
	}

	/**
	 * Get design name by provided design slug
	 *
	 * @param string $design_slug The registered design slug.
	 * @return string
	 */
	public function get_design_name( $design_slug ) {
		$theme_designs = $this->get_theme_designs();
		if ( isset( $theme_designs[ $design_slug ] ) ) {
			return $theme_designs[ $design_slug ]['name'];
		}
		return false;
	}
}

/**
 * Kicking this off by calling 'get_instance()' method
 */
WPZOOM_Onboarding::get_instance();
